# -*- coding: utf-8 -*-

from django.contrib import admin
from django.utils.translation import ugettext as _
from django.utils.translation import ugettext_lazy as ugl
from django import forms

from jumbo.admin.options import ExtendibleModelAdmin, OrderedModelAdmin
from jumbo.forms.models import AdminModelForm
from jumbo.core.urlresolvers import reverse
from jumbo.middleware.thread_local import get_request
from django.conf import settings

from organization.models import *

from organization import views as organization_views
from address.models import Address

### FORM ###
class OrganizationAdminForm(AdminModelForm):
    class Meta:
        model = Organization

    def __init__(self, *args, **kws):
        super(OrganizationAdminForm, self).__init__(*args, **kws)
        category = get_request().GET.get('category', None)
        instance = kws.get('instance', None)
        ## form ADD
        if category and not instance:
            self.fields['organization_type'].queryset = OrganizationType.objects.filter(category=category)
        ## form EDIT
        elif instance:
            category = instance.organization_type.category
            self.fields['organization_type'].queryset = OrganizationType.objects.filter(category=category)


    def clean_vat_number(self):
        data = self.cleaned_data['vat_number']
        if data and len(data) != 11:
            raise forms.ValidationError(_("Vat Number must have length of 11."))
        return data

    def clean_ssn(self):
        from django.contrib.localflavor.it.util import ssn_check_digit
        data = self.cleaned_data['ssn']
        if data and len(data) == 16:
            check = ssn_check_digit(data[0:15])
            if check != data[15]:
                raise forms.ValidationError(_("Invalid SSN code"))
        return data

class ContactAdminForm(AdminModelForm):
    class Meta:
        model = Contact

    def __init__(self, *args, **kws):
        super(ContactAdminForm, self).__init__(*args, **kws)
        category = get_request().GET.get('category', None)
        instance = kws.get('instance', None)
        ## form ADD
        if category and not instance:
            self.fields['organization'].queryset = Organization.objects.filter(organization_type__category=category)
        ## form EDIT
        elif instance:
            try:
                category = instance.organization.organization_type.category
                self.fields['organization'].queryset = Organization.objects.filter(organization_type__category=category)
            except:
                self.fields['organization'].queryset = Organization.objects.all()

    def clean(self):
        cleaned_data = self.cleaned_data
        email                   = cleaned_data.get("email")
        send_procedure_by_email = cleaned_data.get("send_procedure_by_email")
        send_ddt_by_email       = cleaned_data.get("send_ddt_by_email")
        send_invoice_by_email   = cleaned_data.get("send_invoice_by_email")
        if (send_procedure_by_email or send_ddt_by_email or send_invoice_by_email) and not email:
            raise forms.ValidationError("Specify a valid email to send procedure, ddt or invoice")
        return cleaned_data

### INLINE FORM ###
class OrganizationAddressInlineAdminForm(AdminModelForm):
    number = forms.CharField(label=_('number'),
               widget=forms.TextInput(attrs={'size':'10', 'maxlength':'20'})
            )
    zip_code = forms.CharField(label=_('zip_code'),
               widget=forms.TextInput(attrs={'size':'10', 'maxlength':'20'})
            )
    class Meta:
        model = OrganizationAddress

class VectorInlineAdminForm(AdminModelForm):
    class Meta:
        model = M2MtoVector

    def __init__(self, *args, **kws):
        super(VectorInlineAdminForm, self).__init__(*args, **kws)
        self.fields['vector'].queryset = Organization.objects.filter(organization_type__category="vec")

    def clean(self):
        """
        verifica che l'Organizzazione Principale sia diversa dal Vettore
        """
        cleaned_data = self.cleaned_data
        organization = cleaned_data.get("organization")
        vector = cleaned_data.get("vector")
        if organization == vector:
            raise forms.ValidationError(_("Vector have to be different from Organization"))
        if organization.organization_type.category == "vec":
            raise forms.ValidationError(_("Vector can't have Vectors"))
        return cleaned_data

### INLINE ###
class ContactInline(admin.StackedInline):
    model = Contact
    extra = 0
    exclude = ['date_create', 'date_last_modify','creator','last_modifier',]
    can_delete = True
    procedure_fieldset = (
        (_('general information'), {
            'fields': (
                'default',
                'in_addressbook',
                'status',
                'user',
                'language',
                ('organization', 'role',),
                ('last_name', 'first_name'),
                ('phone', 'fax',),
                ('mobile', 'email'),
                'photo',
                'send_quotation_by_email',
                'send_procedure_by_email',
                'send_invoice_by_email',
                'note',
                'ordering'
            )
        }),
    )
    normal_fieldset = (
        (_('general information'), {'fields': (
                ('language', 'role'), ('last_name', 'first_name') ,('phone', 'mobile'), ('email', 'ordering'))
        }),
    )

    def get_fieldsets(self, request, obj=None):
        self.fieldsets = self.normal_fieldset
        if "procedure" in settings.INSTALLED_APPS:
            self.fieldsets = self.procedure_fieldset
        return self.fieldsets


class BankInline(admin.TabularInline):
    model = Bank
    extra = 0
    fieldsets = (
        (_('general information'), {'fields': ('int_code', 'cin', 'abi', 'cab', 'cc', 'note',)}),
    )
    exclude = ['date_create', 'date_last_modify','creator','last_modifier',]
    can_delete = False

class OrganizationAddressInline(admin.StackedInline):
    model = OrganizationAddress
    form = OrganizationAddressInlineAdminForm
    extra = 0
    fieldsets = (
        (_('general information'), {'fields': (
            'address_scopes', ('address_type', 'name'),
            ('number', 'zip_code'), ('locality', 'province'),
            'country', 'note'
        ),}),
    )
    exclude = ['date_create', 'date_last_modify','creator','last_modifier',]
    can_delete = False

class VectorInline(admin.TabularInline):
    model = M2MtoVector
    extra = 0
    form = VectorInlineAdminForm
    fk_name = 'organization'
    fields = ('vector', 'transport_by')
    can_delete = False

class CommisionInline(admin.TabularInline):
    model = Commision
    extra = 0
    max_num = 1
    fields = ('agent', 'commision')
    can_delete = False

if "product" in settings.INSTALLED_APPS:
    from product.models import OrganizationPriceList, OrganizationTypePriceList
    class OrganizationPriceListInline(admin.TabularInline):
        model = OrganizationPriceList
        extra = 0
        max_num = 1
        fields = ('price_list', 'discount1', 'discount2')
        can_delete = False

    class OrganizationTypePriceListInline(admin.TabularInline):
        model = OrganizationTypePriceList
        extra = 0
        max_num = 1
        fields = ('price_list', 'discount1', 'discount2')
        can_delete = True

### SEARCH FORM ###
STATUS = (('', '---'), ('1', ugl('Active')), ('0', ugl('Disactive')),)
class OrganizationAdminSearchForm(forms.ModelForm):
    class Meta:
        model = Organization
        fields = ('organization_type', 'organization_form', 'vat')
        display_fields = (
            ('status', 'organization_type__category',),
            ('code__icontains', 'name__icontains', 'name__istartswith'),
            ('contacts__last_name__icontains', 'contacts__first_name__icontains'),
            ('organization_type', 'organization_form', 'vat'),
            'addresses__locality__icontains'
        )

    def __init__(self, *args, **kws):
        super(OrganizationAdminSearchForm, self).__init__(*args, **kws)
        self.fields['organization_type__category'] = forms.CharField(label=' ', required=False, widget=forms.HiddenInput)
        self.fields['code__icontains']  = forms.CharField(label=_('Code'), required=False)
        self.fields['name__icontains']  = forms.CharField(label=_('Name'), required=False)
        self.fields['name__istartswith'] = forms.CharField(label=_('Starts with'), required=False)
        self.fields['status'] = forms.ChoiceField(label=_("Status"), choices=STATUS, required=False)
        self.fields['contacts__last_name__icontains'] = forms.CharField(label=_('Contact Last name'), required=False)
        self.fields['contacts__first_name__icontains'] = forms.CharField(label=_('Contact First name'), required=False)
        self.fields['addresses__locality__icontains'] = forms.CharField(label=_('Locality'), required=False)

class ContactAdminSearchForm(forms.ModelForm):
    class Meta:
        model = Contact
        fields = ('role',)
        display_fields = (
            ('status', 'organization__organization_type__category'),
            ('role', 'last_name__icontains', 'first_name__icontains'),
            ('organization__name__icontains',),
        )

    def __init__(self, *args, **kws):
        super(ContactAdminSearchForm, self).__init__(*args, **kws)
        self.fields['organization__organization_type__category'] = forms.CharField(label=' ', required=False, widget=forms.HiddenInput)
        self.fields['last_name__icontains'] = forms.CharField(label=_('Last Name'), required=False)
        self.fields['first_name__icontains'] = forms.CharField(label=_('First Name'), required=False)
        self.fields['organization__name__icontains'] = forms.CharField(label=_('Organization'), required=False)
        self.fields['status'] = forms.ChoiceField(label=_("Status"), choices=STATUS, required=False)

### GENERIC ###
class OrganizationAdmin(ExtendibleModelAdmin):
    class Media:
        css = {
            "all" : ("organization/css/admin_organization.css",)
        }

    form = OrganizationAdminForm
    save_on_top = True
    fieldsets = (
        (_('logs admin'), {
            'fields':('creator', 'date_create','last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (_('general information'), {'fields':(
            'status',
            ('code', 'name',),
            ('organization_type', 'organization_form',),
            ('vat', 'delivery'),
            ('payment_form', 'currency'),
            ('vat_number', 'ssn'),
            ('url', 'email'),
            ('phone', 'phone2',),
            'fax',
            'in_addressbook',
        )}),
        (_('Background information admin'), {
            'fields':('note',),
            'classes': ('collapse',)
        }),
    )
    inlines = [ContactInline, OrganizationAddressInline, BankInline, VectorInline, CommisionInline]
    if "product" in settings.INSTALLED_APPS:
        inlines.append(OrganizationPriceListInline)

    filter_horizontal = ('organization_group', )
    list_display = (
        'get_status_icon', 'code', 'get_organization_type', 'name', 'get_phone',
        'get_phone2', 'get_fax_icon', 'get_icon_link_url', 'get_icon_link_email',
        'filter_documents_icon',
        'get_detail_icon', 'get_edit_icon', 'get_delete_icon'
    )
    list_display_csv = (
        'get_status_display', 'code', 'organization_type', 'name', 'phone', 'phone2', 'fax', 'url',
        'email'
    )
    search_form = OrganizationAdminSearchForm
    search_fields = ('code', 'name',)

    def get_urls(self):
        from django.conf.urls.defaults import patterns, url
        urls = super(OrganizationAdmin, self).get_urls()
        my_urls = patterns('',
            url(r'^save_in_addressbook/$'  , self.admin_site.admin_view(organization_views.save_in_addressbook)
                                           , name='organization_organization_save_in_addressbook'),
        )
        return my_urls + urls

    def get_link_url(self, obj):
        if obj.url:
            return """<a href=%s target="_blank">%s</a>""" % (obj.url, obj.url)
        return ""
    get_link_url.short_description = _("Url")
    get_link_url.allow_tags = True

    def get_icon_link_url(self, obj):
        if obj.url:
            return """
                <a href=%(href)s target="_blank">
                    <img src='%(url)sadmin/images/web.png' alt='%(title)s' title='%(title)s'/>
                </a>""" % {
                    'href': obj.url,
                    'url': settings.MEDIA_URL,
                    'title': obj.url
                }
        return ""
    get_icon_link_url.short_description = _("U")
    get_icon_link_url.allow_tags = True

    def get_link_email(self, obj):
        if obj.email:
            return """<a href=mailto:%s>%s</a>""" % (obj.email, obj.email)
        return ""
    get_link_email.short_description = _("Email")
    get_link_email.allow_tags = True

    def get_icon_link_email(self, obj):
        if obj.email:
            return """
                <a href=mailto:%(email)s>
                    <img src='%(url)sadmin/images/email.png' alt='%(title)s' title='%(title)s'/>
                </a>""" % {
                    'email': obj.email,
                    'url': settings.MEDIA_URL,
                    'title': obj.email
                }
        return ""
    get_icon_link_email.short_description = _("E")
    get_icon_link_email.allow_tags = True

    def get_link_organization_detail(self, obj):
        return """<a href='%(reverse_url)s' title='%(title)s'>%(name)s</a>""" % {
            'reverse_url': reverse('admin:organization_organization_detail', obj.id),
            'title': _("View Detail"),
            'name': obj.name
        }
    get_link_organization_detail.short_description = _("Organization")
    get_link_organization_detail.order_admin_field = 'name'
    get_link_organization_detail.allow_tags = True

    def get_organization_type(self, result):
        return "<b>%s</b>" % (result.organization_type.name,)
    get_organization_type.short_description = _("Type")
    get_organization_type.order_admin_field = 'organization_type__category__name'
    get_organization_type.allow_tags = True

    def filter_documents_icon(self, obj):
        return """<a href='%(reverse_url)s?organization__id=%(obj_id)s' title='%(title)s'>x</a>""" % {
            'reverse_url':  reverse('admin:document_document_changelist'),
            'obj_id':       obj.id,
            'title':        _("List Documents")
        }
    filter_documents_icon.short_description = _("D")
    filter_documents_icon.allow_tags = True

    if 'asterisk' in settings.INSTALLED_APPS:
        def get_phone(self, result):
            from asterisk.utils import click_to_call
            return click_to_call(result.phone, get_link=True)
        get_phone.short_description = _("P1")
    else:
        def get_phone(self, result):
            return  result.phone
        get_phone.short_description = _("Phone")
    get_phone.order_admin_field = 'phone'
    get_phone.allow_tags = True

    if 'asterisk' in settings.INSTALLED_APPS:
        def get_phone2(self, result):
            from asterisk.utils import click_to_call
            return click_to_call(result.phone2, get_link=True)
        get_phone2.short_description = _("P2")
    else:
        def get_phone2(self, result):
            return  result.phone2
        get_phone2.short_description = _("Phone 2")
    get_phone2.order_admin_field = 'phone2'
    get_phone2.allow_tags = True

    if 'fax' in settings.INSTALLED_APPS:
        def get_fax_icon(self, obj):
            if obj.fax:
                    #<a href="%(reverse_url)s?recipient_number=%(fax)s" title="Send Fax to %(fax)s">
                    #    <img src="%(url)sadmin/images/fax.png" alt="%(title)s" title="%(title)s"/>
                    #</a>
                return """
                    <a href="%(reverse_url)s" title="Send Fax to %(fax)s">
                        <img src="%(url)sadmin/images/fax.png" alt="%(title)s" title="%(title)s"/>
                    </a>
                """ % {
                        'url': settings.MEDIA_URL,
                        'reverse_url': reverse('admin:fax_fax_add'),
                        'fax': obj.fax,
                        'title': obj.fax
                }
            return ""
        get_fax_icon.short_description = _("F")
    else:
        def get_fax_icon(self, obj):
            return obj.fax
        get_fax_icon.short_description = _("Fax")
    get_fax_icon.order_admin_field = 'fax'
    get_fax_icon.allow_tags = True

class OrganizationGroupAdmin(ExtendibleModelAdmin):
    class Media:
        css = {
            "all" : ("organization/css/admin_organization.css",)
        }

    save_on_top = True
    fieldsets = (
        (_('logs admin'), {
            'fields':('creator', 'date_create','last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (_('general information'), {'fields': ('status', 'name', )}),
    )
    list_display = ('get_status_icon', 'name', 'get_detail_icon', 'get_edit_icon', 'get_delete_icon')
    ordering = ('ordering', 'name')

class OrganizationFormAdmin(ExtendibleModelAdmin):
    class Media:
        css = {
            "all" : ("organization/css/admin_organization.css",)
        }

    save_on_top = True
    fieldsets = (
        (_('logs admin'), {
            'fields':('creator', 'date_create','last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (_('general admin'), {'fields': ('status', 'name')}),
    )
    list_display = ('get_status_icon', 'name', 'ordering', 'get_detail_icon', 'get_edit_icon', 'get_delete_icon')

class OrganizationTypeAdmin(ExtendibleModelAdmin):
    class Media:
        css = {
            "all" : ("organization/css/admin_organization.css",)
        }

    save_on_top = True
    fieldsets = (
        (_('logs admin'), {
            'fields':('creator', 'date_create','last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (_('general information'), {'fields': ('status', 'category', ('code', 'name'), 'description', 'note', 'ordering')}),
    )
    list_display = (
        'get_status_icon', 'code', 'name', 'category', 'ordering', 'get_detail_icon', 'get_edit_icon', 'get_delete_icon'
    )
    ordering = ('ordering', 'code', 'name')

    inlines = []
    if "product" in settings.INSTALLED_APPS:
        inlines.append(OrganizationTypePriceListInline)

class OfficeAdmin(ExtendibleModelAdmin):
    class Media:
        css = {
            "all" : ("organization/css/admin_organization.css",)
        }

    save_on_top = True
    fieldsets = (
        (_('logs admin'), {
            'fields':('creator', 'date_create','last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (_('general information'), {'fields': (('code', 'name'), 'description', 'phone', 'fax', 'email',)}),
    )
    list_display = (
        'get_status_icon', 'code', 'name', 'get_detail_icon', 'get_edit_icon', 'get_delete_icon'
    )
    ordering = ('code', 'name')

class FieldAdmin(ExtendibleModelAdmin):
    class Media:
        css = {
            "all" : ("organization/css/admin_organization.css",)
        }

    save_on_top = True
    fieldsets = (
        (_('logs admin'), {
            'fields':('creator', 'date_create','last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (_('general information'), {'fields': ('status', 'name')}),
    )
    list_display = ('get_status_icon', 'name', 'ordering', 'get_detail_icon', 'get_edit_icon', 'get_delete_icon')
    ordering = ('ordering', 'name',)

class ContactRoleAdmin(ExtendibleModelAdmin):
    class Media:
        css = {
            "all" : ("organization/css/admin_organization.css",)
        }

    save_on_top = True
    fieldsets = (
        (_('logs admin'), {
            'fields':('creator', 'date_create','last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (_('general information'), {'fields': ('status', 'name', 'ordering')}),
    )
    list_display = ('get_status_icon', 'name', 'ordering', 'get_detail_icon', 'get_edit_icon', 'get_delete_icon')
    ordering = ('ordering', 'name')

class ContactAdmin(ExtendibleModelAdmin):
    class Media:
        css = {
            "all" : ("organization/css/admin_organization.css",)
        }

    save_on_top = True
    fieldsets = (
        (_('logs admin'), {
            'fields':('creator', 'date_create','last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (_('general information'), {
            'fields': (
                'default',
                'in_addressbook',
                'status',
                'user',
                'language',
                ('organization', 'role',),
                ('last_name', 'first_name'),
                ('phone', 'fax',),
                ('mobile', 'email'),
                'photo',
                'send_quotation_by_email','send_procedure_by_email', 'send_ddt_by_email', 'send_invoice_by_email',
                'note',
                'ordering'
            )
        }),
    )
    list_display = (
        'get_status_icon', 'last_name', 'first_name', 'role', 'organization',
        'get_phone', 'get_mobile', 'get_fax_icon', 'get_icon_link_email',
        'get_detail_icon', 'get_edit_icon', 'get_delete_icon'
    )
    list_display_csv = (
        'get_status_display', 'last_name', 'first_name', 'role', 'organization', 'phone', 'mobile',
        'fax'
    )
    #raw_id_fields = ('organization',)
    form = ContactAdminForm
    search_form = ContactAdminSearchForm
    search_fields = ('last_name', 'first_name')
    ordering = ('last_name', 'first_name')

    def get_link_email(self, obj):
        if obj.email:
            return """<a href=mailto:%s>%s</a>""" % (obj.email, obj.email)
        return ""
    get_link_email.short_description = _("Email")
    get_link_email.allow_tags = True

    def get_icon_link_email(self, obj):
        if obj.email:
            return """
                <a href=mailto:%(email)s>
                    <img src='%(url)sadmin/images/email.png' alt='%(title)s' title='%(title)s'/>
                </a>
                """ % {
                    'email': obj.email,
                    'url': settings.MEDIA_URL,
                    'title': obj.email
                }
        return ""
    get_icon_link_email.short_description = _("E")
    get_icon_link_email.allow_tags = True

    if 'asterisk' in settings.INSTALLED_APPS:
        def get_phone(self, obj):
            from asterisk.utils import click_to_call
            return click_to_call(obj.phone, get_link=True)
        get_phone.short_description = _("P")
    else:
        def get_phone(self, obj):
            return  obj.phone or ''
        get_phone.short_description = _("Phone")
    get_phone.order_admin_field = 'phone'
    get_phone.allow_tags = True

    if 'asterisk' in settings.INSTALLED_APPS:
        def get_mobile(self, obj):
            from asterisk.utils import click_to_call
            return click_to_call(obj.mobile, get_link=True)
        get_mobile.short_description = _("M")
    else:
        def get_mobile(self, obj):
            return  obj.mobile or ''
        get_mobile.short_description = _("Mobile")
    get_mobile.order_admin_field = 'mobile'
    get_mobile.allow_tags = True

    if 'fax' in settings.INSTALLED_APPS:
        def get_fax_icon(self, obj):
            if obj.fax:
                    #<a href="%(reverse_url)s?recipient_number=%(fax)s" title="Send Fax to %(fax)s">
                    #    <img src="%(url)sadmin/images/fax.png" alt="%(title)s" title="%(title)s"/>
                    #</a>
                return """
                    <a href="%(reverse_url)s" title="Send Fax to %(fax)s">
                        <img src="%(url)sadmin/images/fax.png" alt="%(title)s" title="%(title)s"/>
                    </a>
                """ % {
                        'url': settings.MEDIA_URL,
                        'reverse_url': reverse('admin:fax_fax_add'),
                        'fax': obj.fax,
                        'title': obj.fax
                }
            return ""
        get_fax_icon.short_description = _("F")
    else:
        def get_fax_icon(self, obj):
            return obj.fax or ''
        get_fax_icon.short_description = _("Fax")
    get_fax_icon.order_admin_field = 'fax'
    get_fax_icon.allow_tags = True

class BankAdmin(ExtendibleModelAdmin):
    class Media:
        css = {
            "all" : ("organization/css/admin_organization.css",)
        }

    save_on_top = True
    fieldsets = (
        (_('logs admin'), {
            'fields':('creator', 'date_create','last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (_('general information'), {'fields': ('status', 'organization', 'int_code', 'cin', 'abi', 'cab', 'cc')}),
    )
    list_display = (
        'get_status_icon', 'organization', 'int_code', 'cin', 'abi', 'cab', 'cc', 'get_detail_icon', 'get_edit_icon', 'get_delete_icon'
    )

class VatAdmin(ExtendibleModelAdmin, OrderedModelAdmin):
    class Media:
        css = {
            "all" : ("organization/css/admin_organization.css",)
        }

    save_on_top = True
    fieldsets = (
        (_('logs admin'), {
            'fields':('creator', 'date_create','last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (_('general information'), {'fields': ('status','name','code','description',),}),
        (_('aliq admin'), {'fields': ('aliquot_perc','deduct_perc',),}),
    )
    search_fields = ('name','code')
    list_display = (
        'get_status_icon', 'name', 'code', 'aliquot_perc', 'deduct_perc', 'get_detail_icon', 'get_edit_icon', 'get_delete_icon'
    )

class CurrencyAdmin(ExtendibleModelAdmin, OrderedModelAdmin):
    class Media:
        css = {
            "all" : ("organization/css/admin_organization.css",)
        }

    save_on_top = True
    fieldsets = (
        (_('logs admin'), {
            'fields':('creator', 'date_create','last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (_('general information'), {'fields': ('status','name','code','symbol',),}),
    )
    search_fields = ('name','code')
    list_display = (
        'get_status_icon', 'name', 'code', 'symbol', 'get_detail_icon', 'get_edit_icon', 'get_delete_icon'
    )

class DeliveryAdmin(ExtendibleModelAdmin, OrderedModelAdmin):
    class Media:
        css = {
            "all" : ("organization/css/admin_organization.css",)
        }

    save_on_top = True
    fieldsets = (
        (_('logs admin'), {
            'fields':('creator', 'date_create','last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (_('general information'), {'fields': ('status','transport_by','carriage',),}),
    )
    search_fields = ('transport_by','carriage')
    list_display = (
        'get_status_icon', 'transport_by', 'carriage', 'get_detail_icon', 'get_edit_icon', 'get_delete_icon'
    )

class AgentAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("organization/css/admin_organization.css",)}

    save_on_top = True
    fieldsets = (
        (_('logs admin'), {
            'fields': ('creator', 'date_create','last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (_('general admin'), {'fields': ('status', 'code', 'name', 'user',
                                         'email', 'mobile', 'area',
                                         'is_area_manager', 'commision'),}),
    )
    search_fields = ('code','name',)
    ordering = ('name',)
    list_display = ('get_status_icon', 'code', 'name', 'get_link_email', 'mobile', 'commision',
                    'get_detail_icon', 'get_edit_icon', 'get_delete_icon')

    def get_link_email(self, obj):
        if obj.email:
            return """<a href=mailto:%s>%s</a>""" % (obj.email, obj.email)
        return ""
    get_link_email.short_description = _("Email")
    get_link_email.allow_tags = True

class CommisionAdminSearchForm(forms.ModelForm):
    class Meta:
        model = Commision
        display_fields = (
            ('agent__code', 'agent__name',),
            ('organization__code', 'organization__name'),
        )

    def __init__(self, *args, **kws):
        super(CommisionAdminSearchForm, self).__init__(*args, **kws)
        self.fields['agent__code'] = forms.CharField(label=_('Agent Code'), required=False)
        self.fields['agent__name']  = forms.CharField(label=_('Agent Name'), required=False)
        self.fields['organization__code']  = forms.CharField(label=_('Organization Name'), required=False)
        self.fields['organization__name'] = forms.CharField(label=_('Organization Code'), required=False)

class CommisionAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("organization/css/admin_organization.css",)}

    save_on_top = True
    fieldsets = (
        (_('logs admin'), {'fields': ('creator', 'date_create','last_modifier', 'date_last_modify',), 'classes': ('collapse',)}),
        (_('general admin'), {'fields': ('status','agent', 'organization', 'commision'),}),
    )
    search_form = CommisionAdminSearchForm
    search_fields = ('=agent__code', 'agent__name', '=organization__code', 'organization__name')
    ordering = ('agent__code', 'organization__code',)
    list_display = ('get_status_icon', 'get_agent_detail', 'get_organization_detail', 'commision', 'get_detail_icon', 'get_edit_icon', 'get_delete_icon')
    actions = ['reset_commision',]

    def reset_commision(self, request, queryset):
        """ Reset to 0 the agent's organizations commisions """
        rows_updated = queryset.update(commision=0)
        if rows_updated == 1:
            message_bit = _("1 commision was")
        else:
            message_bit = _("%s commisions were") % rows_updated
        self.message_user(request, _("%s successfully reset.") % message_bit)
    reset_commision.short_description = _("Reset Commision")

    def get_agent_detail(self, obj):
        return obj.agent.name
    get_agent_detail.short_description = _("Agent")
    get_agent_detail.admin_order_field = 'agent__name'
    get_agent_detail.allow_tags = True

    def get_organization_detail(self, obj):
        return obj.organization.name
    get_organization_detail.short_description = _("Organization")
    get_organization_detail.admin_order_field = 'organization__name'
    get_organization_detail.allow_tags = True


class CarriageAdmin(ExtendibleModelAdmin):
    fieldsets = (
        (_('general information'), {'fields': ('code','name','description',),}),
    )
    search_fields = ('code','name')
    list_display = (
        'get_status_icon', 'code', 'name', 'description', 'get_edit_icon', 'get_delete_icon'
    )

class OrganizationAddressAdmin(ExtendibleModelAdmin):
    search_fields = ('name', 'locality')
    list_display = (
        'get_status_icon', 'get_scopes', 'organization',
        'get_address_type_display', 'name', 'number',
        'zip_code', 'locality', 'province', 'country',
        'get_detail_icon', 'get_edit_icon', 'get_delete_icon'
    )

admin.site.register(Organization, OrganizationAdmin)
admin.site.register(OrganizationGroup, OrganizationGroupAdmin)
admin.site.register(OrganizationForm, OrganizationFormAdmin)
admin.site.register(OrganizationType, OrganizationTypeAdmin)
admin.site.register(Office, OfficeAdmin)
admin.site.register(Field, FieldAdmin)
admin.site.register(ContactRole, ContactRoleAdmin)
admin.site.register(Contact, ContactAdmin)
admin.site.register(Bank, BankAdmin)
admin.site.register(Vat, VatAdmin)
admin.site.register(Currency, CurrencyAdmin)
admin.site.register(Delivery, DeliveryAdmin)
admin.site.register(Agent, AgentAdmin)
admin.site.register(Commision, CommisionAdmin)
admin.site.register(Carriage, CarriageAdmin)
admin.site.register(OrganizationAddress, OrganizationAddressAdmin)
