# -*- coding: utf-8 -*-
from datetime import date, timedelta

from django.conf import settings
from django.db import models, connection
from django.db.models import Avg, Sum, Min, Max
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ugettext
from django.contrib.auth.models import User
from django.core.exceptions import ValidationError

from jumbo.db.models import DateModel, UserModel, StatusModel, OrderedModel, DefaultModel
from jumbo.apps.abicab.models import Bank as DetailBank
from jumbo.apps.i18n.models import Language
from jumbo.utils.functions import get_settings

from address.models import Address
from payment.models import PaymentForm


#############################
# UFFICIO                   #
#############################
class Office (DateModel, UserModel, StatusModel):
    ## ForeignKey
    ## Campi
    code = models.CharField (
        unique = True,
        max_length = 15,
        verbose_name = _("code")
    )
    name = models.CharField (
        unique = True,
        max_length = 50,
        verbose_name = _("office")
    )
    description = models.TextField (
        blank = True, null = True,
        max_length = 255,
        verbose_name = _("description")
    )
    phone = models.CharField (
        max_length = 20,
        verbose_name = _("phone"),
    )
    fax = models.CharField (
        max_length = 20,
        verbose_name = _("fax"),
    )
    email = models.EmailField (
        max_length = 100,
        blank = True, null = True,
        verbose_name = _("email"),
    )

    class Meta:
        verbose_name = _("office")
        verbose_name_plural = _("offices")

    def __unicode__(self):
        return u"%s" % self.name

################################
# ANAGRAFICA                   #
################################
class OrganizationAddress (Address):
    organization = models.ForeignKey (
        "Organization",
        db_index = True,
        related_name='addresses',
        verbose_name = _("organization"),
    )
    ## il nome della società a cui l'indirizzo si riferisce potrebbe essere diversa da 'Organization'
    org_name = models.CharField (
        max_length = 100,
        blank = True, null = True,
        db_index = True,
        verbose_name = _("Organization Name"),
    )
    note = models.TextField (
        blank = True, null = True,
        verbose_name = _("note"),
    )

    class Meta:
        verbose_name = _("organization address")
        verbose_name_plural = _("organization addresses")

class Organization (DateModel, UserModel, StatusModel, OrderedModel):
    ## ForeignKey
    office = models.ForeignKey (
        Office,
        blank = True, null = True,
        db_index = True,
        related_name = "organizations",
        verbose_name = _("office"),
    )
    organization_group = models.ManyToManyField(
        "OrganizationGroup",
        blank = True, null = True,
        db_index = True,
        related_name = "organizations",
        verbose_name = _("group"),
    )
    organization_type = models.ForeignKey (
        "OrganizationType",
        db_index = True,
        related_name = "organizations",
        verbose_name = _("Organization type"),
    )
    organization_form = models.ForeignKey (
        "OrganizationForm",
        db_index = True,
        related_name = "organizations",
        verbose_name = _("Organization form"),
    )
    field = models.ForeignKey (
       "Field",
        blank = True, null = True,
        db_index = True,
        related_name = "organizations",
        verbose_name = _("Field"),
    )
    payment_form = models.ForeignKey (
        PaymentForm,
        blank = True, null = True,
        db_index = True,
        related_name = "organizations",
        verbose_name = _('Payment form'),
    )
    currency = models.ForeignKey (
        "Currency",
        blank = True, null = True,
        related_name = "organizations",
        db_index = True,
        verbose_name = _("Currency"),
    )
    delivery = models.ForeignKey (
        "Delivery",
        blank = True, null = True,
        related_name = "organizations",
        db_index = True,
        verbose_name = _("Delivery"),
    )
    vat = models.ForeignKey (
        "Vat",
        blank = True, null = True,
        related_name = "organizations",
        db_index = True,
        verbose_name = _("Vat"),
    )
    ## Campi
    code = models.CharField (
        unique = True,
        max_length = 15,
        db_index = True,
        verbose_name = _("code"),
    )
    name = models.CharField (
        max_length = 100,
        db_index = True,
        verbose_name = _("name"),
    )
    brand = models.CharField (
        max_length = 255,
        blank = True, null = True,
        db_index = True,
        verbose_name = _("brand"),
    )
    ssn = models.CharField (
        max_length = 16,
        blank = True, null = True,
        db_index = True,
        verbose_name = _("ssn"),
    )
    vat_number = models.CharField (
        max_length = 14,
        blank = True, null = True,
        db_index = True,
        verbose_name = _("vat number"),
    )
    url = models.URLField (
        max_length=100,
        blank = True, null = True,
        verify_exists = True,
        verbose_name = _("url"),
    )
    email = models.EmailField (
        max_length = 100,
        blank = True, null = True,
        verbose_name = _("email"),
    )
    phone = models.CharField (
        max_length = 20,
        verbose_name = _("phone"),
    )
    phone2 = models.CharField (
        blank = True, null = True,
        max_length = 20,
        verbose_name = _("phone2"),
    )
    fax = models.CharField (
        blank = True, null = True,
        max_length = 20,
        verbose_name = _("fax"),
    )
    note = models.TextField (
        blank = True, null = True,
        verbose_name = _("note"),
    )
    in_addressbook = models.NullBooleanField(
        blank = True, null = True,
        verbose_name = _("in rubrica"),
        help_text = _("If this number is in the phone address book.")
    )
    newsletter_subscribed = models.BooleanField(
        default = False,
        verbose_name = _("newsletter sottoscritta"),
        help_text = _("If this organization have a newsletter subription")
    )

    class Meta:
        # FIXME
        # unique_together = (("vat_number", "ssn", "organization_type__category"),)
        unique_together = (("vat_number", "ssn"),)
        verbose_name = _("organization")
        verbose_name_plural = _("organizations")
        permissions = (
            ("view_organization"    , "Can view organizations"),
            ("save_addressbook"     , "Can save in phone addreebook"),
        )
        ordering = ['name']

    def __unicode__(self):
        return u"%s" % self.name

    def save(self):
        ## Se l'anagrafica è di tipo vettore non possono essere agganciati altri vettori
        if self.organization_type.category == "vec" and self.vectors.all():
            raise  ValidationError("Vector can't have Vectors")

        ## modifico il valore del name
        osn = get_settings("organization.settings", "ORGANIZATION_NAME_STYLE")
        if osn == "title":
            self.name = self.name.title()
        elif osn == "lower":
            self.name = self.name.lower()
        elif osn == "upper":
            self.name = self.name.upper()

        super(Organization, self).save()


    def get_addresses(self):
        return self.addresses.filter(status=1)

    def get_first_address(self):
        try:
            return self.get_addresses()[0]
        except:
            return None

    def get_shipping_addresses(self):
        addresses = self.get_addresses()
        shipping_addresses = []
        for address in addresses:
            is_shipping_addresses=address.address_scopes.all().filter(is_shipping_address=True)
            if is_shipping_addresses:
                shipping_addresses.append(address)
        return shipping_addresses

    def get_invoice_addresses(self):
        addresses = self.get_addresses()
        invoice_addresses = []
        for address in addresses:
            is_invoice_addresses=address.address_scopes.all().filter(is_invoice_address=True)
            if is_invoice_addresses:
                invoice_addresses.append(address)
        return invoice_addresses

    def default_invoice_address1(self):
        try:
            addr = self.get_invoice_addresses()[0]
            return "%s %s, %s" % (addr.get_address_type_display() or "", addr.name, addr.number)
        except:
            return None

    def default_invoice_address2(self):
        try:
            addr = self.get_invoice_addresses()[0]
            return "%s %s (%s)" % (addr.zip_code, addr.locality, addr.province)
        except:
            return None

    def default_invoice_address3(self):
        try:
            addr = self.get_invoice_addresses()[0]
            return "%s" % addr.country
        except:
            return None

    def get_agents(self):
        return self.agent.filter(status=1)

    def get_commisions(self):
        return self.commisions.all()

    def get_commision(self):
        try:
            return self.get_commisions()[0]
        except:
            return None

    def get_contacts(self, other_filters={}):
        c = self.contacts.filter(status=1)
        if other_filters:
            c = c.filter(**other_filters)
        return c

    def get_quotation_contacts(self):
        contact_filter = {'send_quotation_by_email':True}
        return self.get_contacts(other_filters=contact_filter)

    def get_procedure_contacts(self):
        contact_filter = {'send_procedure_by_email':True}
        return self.get_contacts(other_filters=contact_filter)


    def get_ddt_contacts(self):
        contact_filter = {'send_ddt_by_email':True}
        return self.get_contacts(other_filters=contact_filter)


    def get_invoice_contacts(self):
        contact_filter = {'send_invoice_by_email':True}
        return self.get_contacts(other_filters=contact_filter)


    def get_agents(self):
        return self.agent.filter(status=1)


    def get_all_mails_for_procedure(self):
        contact_filter = {'send_procedure_by_email':True}
        return self.get_all_mails(contact_filter=contact_filter)


    def get_all_mails_for_ddt(self):
        contact_filter = {'send_ddt_by_email':True}
        return self.get_all_mails(contact_filter=contact_filter)


    def get_all_mails_for_invoice(self):
        contact_filter = {'send_invoice_by_email':True}
        return self.get_all_mails(contact_filter=contact_filter)

    def get_all_mails_for_quotation(self):
        contact_filter = {'send_quotation_by_email':True}
        return self.get_all_mails(contact_filter=contact_filter)

    def get_all_mails(self, contact_filter={}):
        """ Return all organizations and their contacts mails"""
        mails = []

        # Organization Mail
        if self.email:
            label_mail = ugettext("%(email)s (Organization)" % {'email':self.email})
            choice_mail = (self.email, label_mail)
            if choice_mail not in mails:
                mails.append(choice_mail)

        # All contacts mail
        for c in self.get_contacts(other_filters=contact_filter):
            if c.email:
                label_mail = ugettext("%(email)s (Contact)" % {'email':c.email})
                choice_mail = (c.email, label_mail)
                if choice_mail not in mails:
                    mails.append(choice_mail)

        # All agents mail
        for a in self.get_agents():
            if a.email:
                label_mail = ugettext("%(email)s (Agent)" % {'email':a.email})
                choice_mail = (a.email, label_mail)
                if choice_mail not in mails:
                    mails.append(choice_mail)

        return mails

    if "agenda" in settings.INSTALLED_APPS:
        from django.contrib.contenttypes import generic
        from agenda.models import Membership
        memberships = generic.GenericRelation(Membership)

        def get_appointments(self):
            memberships = self.memberships.all()
            appointments = {}
            for m in memberships:
                if not appointments.has_key(m.appointment.id):
                    appointments[m.appointment.id] = m.appointment
            return appointments.values()

    if "commision" in settings.INSTALLED_APPS:
        def get_balance(self, date_b = date.today()):
            """
            restituisce il saldo dell'Anagrafica
            alla data specificata
            """
            cursor = connection.cursor()
            sql = """
                SELECT SUM(amount) AS balance
                FROM commision_commision
                WHERE organization_id = %s
                AND operation_date <= %s
            """
            params = [self.id, date_b.strftime('%Y-%m-%d')]
            cursor.execute(sql, params)
            row = cursor.fetchone()
            if row[0] is None:
                return 0.00
            else:
                return row[0]

    if "jumbo.apps.commision" in settings.INSTALLED_APPS:
        def is_insolvent(self, date_b = date.today()):
            """
            restituisce True o False se il cliente ha un saldo negativo alla data specificata
            """
            if self.get_balance(date_b) < 0: return True
            return False

    if "procedure" in settings.INSTALLED_APPS:
        def get_procedurerows(self, date_begin = date.today(), date_end = date.today()):
            """
            restituisce i prodotti delle Pratiche in un certo periodo
            """
            procedures = self.procedures.filter(date_p__gte=date_begin, date_p__lte=date_end)
            total_rows = []
            for procedure in procedures:
                rows = procedure.rows.all()
                total_rows.extend(rows)
            return total_rows

    if "ddt" in settings.INSTALLED_APPS:
        def get_ddtrows(self, date_begin = date.today(), date_end = date.today()):
            """
            restituisce i prodotti dei DDT in un certo periodo
            """
            ddts = self.ddts.filter(date_d__gte=date_begin, date_d__lte=date_end)
            total_rows = []
            for ddt in ddts:
                rows = ddt.rows.all()
                total_rows.extend(rows)
            return total_rows

    if "ddt" in settings.INSTALLED_APPS:
        def get_ddtrows_todelivery(self, date_begin = date.today(), date_end = date.today()):
            """
            restituisce i prodotti da consegnare in
            un certo periodo di tempo
            FIXME "to delivery"
            """
            ddts = self.ddts.filter(date_d__gte=date_begin, date_d__lte=date_end)
            total_rows = []
            for ddt in ddts:
                rows = ddt.rows.all()
                total_rows.extend(rows)
            return total_rows

    if "invoice" in settings.INSTALLED_APPS:
        def get_invoicerows(self, date_begin = date.today(), date_end = date.today()):
            """
            restituisce i prodotti fatturati in un certo periodo
            """
            invoices = self.owner_invoices.filter(date_i__gte=date_begin, date_i__lte=date_end)
            total_rows = []
            for invoice in invoices:
                rows = invoice.rows.all()
                total_rows.extend(rows)
            return total_rows

    if "invoice" in settings.INSTALLED_APPS:
        def get_total_invoiced(self, year = None):
            if not year:
                year = date.today().year
            totals = self.owner_invoices.filter(date_i__year="%s" % year).aggregate(Sum('total_income'), Sum('total_fee'), Sum('total_discount'), Sum('total_vat'))
            totals['year'] = year
            try:
                totals['totals__sum'] = totals['total_income__sum'] + totals['total_fee__sum'] - totals['total_discount__sum'] + totals['total_vat__sum']
            except:
                totals['totals__sum'] = None
            return totals

        def get_year_invoiced(self):
            l = []
            year = date.today().year
            for y in [year, year-1, year-2]:
                l.append(self.get_total_invoiced(y))
            return l

    if "procedure" in settings.INSTALLED_APPS:
        def get_total_ordered(self, year = None):
            if not year:
                year = date.today().year
            totals = self.owner_procedures.filter(date_p__year="%s" % year).aggregate(Sum('total_income'), Sum('total_fee'), Sum('total_discount'), Sum('total_vat'))
            totals['year'] = year

            # Aggiungo al totale degli ordini anche un eventuale totale ordinato (o fatturato,
            # qua e' da capire bene, xke x mentasti il fatturato e' l'ordinato) dell'anno
            # che rappresenta un'eventuale quantita iniziale settata se si inizia ad usare il
            # programma a meta' anno
            #
            total_invoice = self.total_invoiced.filter(year=year)
            if total_invoice:
                total_invoice = total_invoice[0].total
            else:
                total_invoice = 0

            try:
                totals['totals__sum'] = (totals['total_income__sum'] or 0) + \
                                        (totals['total_fee__sum'] or 0) - \
                                        (totals['total_discount__sum'] or 0) + \
                                        (totals['total_vat__sum'] or 0) + \
                                        total_invoice
            except Exception, e:
                totals['totals__sum'] = None
            return totals

        def get_year_ordered(self, num_year=4):
            l = []
            first_year = date.today().year - num_year + 1
            last_year = date.today().year + 1

            for y in reversed(range(first_year, last_year)):
                l.append(self.get_total_ordered(y))
            return l

    if "suspension" in settings.INSTALLED_APPS:
        def get_stops(self, date_s = date.today()):
            """
            restituisce tutte le sospensioni dell'Anagrafica alla data specificata
            """
            stop = self.suspensions.filter(date_begin__lte=date_s, date_end__gte=date_s)
            return stop

    if "suspension" in settings.INSTALLED_APPS:
        def is_stopped(self, date_s = date.today()):
            """
            restituisce True o False se il Cliente e' sospeso nel giorno specificato
            """
            stop = self.suspensions.filter(date_begin__lte=date_s, date_end__gte=date_s)[0]
            if stop: return True
            return False

    if "suspension" in settings.INSTALLED_APPS:
        def is_stopped_in(self, date_begin = date.today(), date_end = date.today()):
            """
            restituisce True o False se eiste già una sospensione dell'Anagrafica nel periodo specificato.
            """
            stop = self.suspensions.filter(date_begin__lte=date_end, date_end__gte=date_begin)[0]
            if stop: return True
            return False

    if "suspension" in settings.INSTALLED_APPS:
        def is_forever_stopped(self, date_s = date.today()):
            """
            restituisce True o False se il Cliente e' sospeso definitivamente:
            e' sospeso definitivamente se non esiste la data di fine
            """
            stop = self.suspensions.filter(date_begin__lte=date_s, date_end__isnull=True)[0]
            if stop: return True
            return False

    def is_credit_card_expired(self, date_e = date.today(), days = timedelta(days = 30)):
        """
        restituisce True o False se il cliente ha carta di credito sospesa:
        """
        pass

    def set_office (self, office):
        """
        assegna o modifica ufficio nelle Pratiche Beneficiario e nell'Anagrafica
        """
        pass

    if "suspension" in settings.INSTALLED_APPS:
        def set_start(self, cause, date_end = date.today()):
            """
            crea una sospensione con data inizio = NULL e data fine = giorno precedente a quello impostato
            L'anagrafica è attiva dal giorno impostato
            """
            from suspension.models import Suspension
            suspension = Suspension(organization=self, disable_cause=cause, date_begin=None, date_end=date_end+timedelta(days=-1))
            suspension.save()

    if "suspension" in settings.INSTALLED_APPS:
        def set_restart(self, date_end = date.today()):
            """
            attiva l'Anagrafica disattivata da una sospensione modificando la
            data di fine sospensione inserendo il giorno precedente.
            La data passata deve essere compresa nell'intervallo di sospensione.
            """
            suspensions = self.get_stops(date_end)
            for suspension in suspensions:
                suspension.date_end = date_end+timedelta(days=-1)
                suspension.save()

    if "suspension" in settings.INSTALLED_APPS:
        def set_stop(self, cause, date_begin = date.today(), date_end = None):
            """
            disattiva il cliente creando una sospensione dalla data di inizio
            alla data di fine specificate inserendo una causale.
            Se la data di fine è NULL l'Anagrafica è sospesa definitivamente.
            Non verifica se c'è una sovrapposizione di sospensioni
            CONTROLLI:
            - La data di inizio deve essere <= alla data di fine
            - La data di inizio deve essere > della data di ultima consegna (get_last_ddt())
            """
            if not cause:
                return
            if date_end and date_begin > date_end:
                return
            from ddt.utils import get_last_ddt
            last_ddt = get_last_ddt()
            if date_begin <= last_ddt.date_d: return
            from suspension.models import Suspension
            suspension = Suspension(organization=self, disable_cause=cause, date_begin=date_begin, date_end=date_end)
            suspension.save()


TRANSPORTTYPE_DEF = (
    ("air", _("air"),),
    ("bot", _("boat"),),
    ("trn", _("train"),),
    ("van", _("van"),),
)

class M2MtoVector(models.Model):
    organization = models.ForeignKey(
        Organization,
        verbose_name = _("Organization"),
        related_name = "vectors"
    )
    vector = models.ForeignKey(
        Organization,
        verbose_name = _("Vector"),
        related_name = "organizations"
    )
    transport_by = models.CharField(
        max_length = 3,
        choices = TRANSPORTTYPE_DEF,
        db_index = True,
        verbose_name = _("Transport by"),
    )

    class Meta:
        verbose_name = _("Vector")
        verbose_name_plural = _("Vectors")
        unique_together = (("organization", "vector"),)

    def __unicode__(self):
        return u"%s" % self.vector.name

    #def save(self):
        #if self.organization == self.vector:
            #raise ValidationError("Vector have to be different from Organization")
        #if self.organization.organization_type.category == "vec":
            #raise ValidationError("Select a valid Organization")
        #if self.vector.organization_type.category != "vec":
            #raise ValidationError("Select a valid Vector")
        #super(M2MtoVector, self).save()

##################################
# GRUPPO SOCIETA                 #
##################################
class OrganizationGroup (DateModel, UserModel, StatusModel, OrderedModel):
    ## ForeignKey
    ## Campi
    name = models.CharField (
        unique = True,
        max_length = 100,
        db_index = True,
        verbose_name = _("name"),
    )

    class Meta:
        verbose_name = _("organization group")
        verbose_name_plural = _("organization group")

    def __unicode__(self):
        return u"%s" % self.name

##################################
# FORMA SOCIETA                  #
##################################
class OrganizationForm (DateModel, UserModel, StatusModel, OrderedModel):
    ## ForeignKey
    ## Campi
    name = models.CharField (
        unique = True,
        max_length = 100,
        db_index = True,
        verbose_name = _("name"),
    )
    description = models.CharField (
        max_length = 100,
        blank=True,
        db_index = True,
        verbose_name = _('description'),
    )

    class Meta:
        verbose_name = _("organization form")
        verbose_name_plural = _("organization forms")
        ordering = ("ordering", "name")

    def __unicode__(self):
        return u"%s" % self.name

####################################
# TIPO ANAGRAFICA                  #
####################################
ORGANIZATIONTYPE_DEF = (
    ("com", _("company"),),
    ("cus", _("customer"),),
    ("sup", _("supplier"),),
    ("vec", _("vector"),),
    ("wha", _("warehouse"),),
    ("oth", _("other"),)
)

class OrganizationType (DateModel, UserModel, StatusModel, OrderedModel):
    ## ForeignKey

    ## Campi
    code = models.CharField (
        unique = True,
        max_length = 15,
        db_index = True,
        verbose_name = _("code"),
    )
    category = models.CharField (
        max_length = 15,
        choices = ORGANIZATIONTYPE_DEF,
        db_index = True,
        verbose_name = _("category"),
    )
    name = models.CharField (
        unique = True,
        max_length = 100,
        db_index = True,
        verbose_name = _("name"),
    )
    description = models.TextField (
        blank = True, null = True,
        verbose_name = _("description"),
    )
    note = models.TextField (
        blank = True, null = True,
        verbose_name = _("background informations"),
    )

    class Meta:
        verbose_name = _("organization type")
        verbose_name_plural = _("organization types")
        ordering = ('ordering', 'name')

    def __unicode__(self):
        return u"%s" % self.name

#############################
# SETTORE                   #
#############################
class Field (DateModel, UserModel, StatusModel, OrderedModel):
    ## ForeignKey
    ## Campi
    name = models.CharField (
        unique = True,
        max_length = 100,
        db_index = True,
        verbose_name = _("name"),
    )

    class Meta:
        verbose_name = _("field")
        verbose_name_plural = _("fields")

    def __unicode__(self):
        return u"%s" % self.name

###################################
# CONTATTO RUOLO                  #
###################################
class ContactRole (DateModel, UserModel, StatusModel, OrderedModel):
    ## ForeignKey
    ## Campi
    name = models.CharField (
        unique = True,
        max_length = 100,
        db_index = True,
        verbose_name = _("name"),
    )

    class Meta:
        verbose_name = _("Contact role")
        verbose_name_plural = _("Contact roles")
        ordering = ('name',)

    def __unicode__(self):
        return u"%s" % self.name

##############################
# CONTATTO                   #
##############################
class Contact (DateModel, UserModel, StatusModel, DefaultModel, OrderedModel):
    ## ForeignKey
    user = models.OneToOneField (
        User,
        blank = True, null = True,
        db_index = True,
        related_name='contact',
        verbose_name = _("User"),
    )
    organization = models.ForeignKey (
        Organization,
        blank = True, null = True,
        db_index = True,
        related_name='contacts',
        verbose_name = _("Organization"),
    )
    role = models.ForeignKey (
        ContactRole,
        blank = True, null = True,
        related_name='contacts',
        verbose_name = _("Title"),
    )
    language = models.ForeignKey(
        Language,
        blank = True, null = True,
        db_index = True,
        verbose_name=_('language'),
        help_text=_('language'),
    )
    ## Campi
    code = models.CharField (
        max_length = 15,
        db_index = True,
        null=True, blank=True,
        verbose_name = _("code"),
    )
    last_name = models.CharField (
        max_length = 100,
        null=True, blank=True,
        verbose_name = _("Last Name"),
    )
    first_name = models.CharField (
        max_length = 100,
        blank = True, null = True,
        verbose_name = _("First Name"),
    )
    phone = models.CharField (
        max_length = 20,
        verbose_name = _("Phone"),
        blank = True, null = True
    )
    mobile = models.CharField (
        max_length = 20,
        blank = True, null = True,
        verbose_name = _("Mobile"),
    )
    fax = models.CharField (
        max_length = 20,
        blank = True, null = True,
        verbose_name = _("Fax"),
    )
    email = models.EmailField (
        max_length = 100,
        blank = True, null = True,
        verbose_name = _("Email"),
    )
    note = models.TextField (
        max_length = 255,
        blank = True, null = True,
        verbose_name = _("Note"),
    )
    in_addressbook = models.NullBooleanField(
        blank = True, null = True,
        verbose_name = _("in rubrica"),
        help_text = _("If this number is in the phone address book.")
    )
    ## se i campi successivi sono 'flaggati' il campo 'email' diventa obbligatorio
    send_procedure_by_email = models.BooleanField(
        default = False,
        verbose_name = _("Send Procedure by email"),
    )
    send_ddt_by_email = models.BooleanField(
        default = False,
        verbose_name = _("Send DDT by email"),
    )
    send_invoice_by_email = models.BooleanField(
        default = False,
        verbose_name = _("Send Invoice by email"),
    )
    send_quotation_by_email = models.BooleanField(
        default = False,
        verbose_name = _("Send Quotation by email"),
    )
    photo = models.ImageField(
        upload_to = get_settings("organization.settings", "CONTACT_PHOTO"),
        blank = True, null = True,
        verbose_name = _("Photo"),
    )

    def get_full_name(self):
        return "%s %s" % (self.last_name, self.first_name)
    full_name = property(get_full_name)

    class Meta:
        verbose_name = _("contact")
        verbose_name_plural = _("contacts")
        permissions = (
            ("view_contact"   , "Can view contacts"),
        )
        ordering = ['ordering', 'last_name', 'first_name']

    def save(self):
        ## modifico il valore del last_name e del first_name
        osn = get_settings("organization.settings", "CONTACT_NAME_STYLE")
        if osn == "title":
            self.last_name = self.last_name.title()
            self.first_name = self.first_name.title()
        elif osn == "lower":
            self.last_name = self.last_name.lower()
            self.first_name = self.first_name.lower()
        elif osn == "upper":
            self.last_name = self.last_name.upper()
            self.first_name = self.first_name.upper()

        super(Contact, self).save()

    if "shop" in settings.INSTALLED_APPS:
        """
        restituisce gli stati del carrello
        """

        def get_basket_status(self):
            baskets = self.basket.all()
            if not baskets:
                return "empty"
            for basket in baskets:
                if basket.product.shop.fill_basket:
                    return "full"
            return "load"

    def __unicode__(self):
        return u"%(last_name)s %(first_name)s (%(role)s)" % {'last_name': self.last_name, 'first_name': self.first_name, 'role': self.role}

##############################
# BANCA                      #
##############################
class Bank(DateModel, UserModel, StatusModel):
    ## ForeignKey
    organization = models.ForeignKey (
        Organization,
        related_name = 'banks',
        verbose_name = _("organization"),
    )
    ## Campi Modello
    abi = models.CharField (
        max_length = 5,
        db_index = True,
        verbose_name = _("abi"),
    )
    cab = models.CharField (
        max_length = 5,
        db_index = True,
        verbose_name = _("cab"),
    )
    int_code = models.CharField (
        max_length = 4,
        blank = True,
        null = True,
        verbose_name = _("Int code")
    )
    cin = models.CharField (
        max_length = 1,
        blank = True,
        null = True,
        verbose_name = _("CIN")
    )
    cc = models.CharField (
        max_length = 12,
        blank = True,
        null = True,
        verbose_name = _("CC")
    )
    note = models.TextField (
        max_length = 255,
        blank = True, null = True,
        verbose_name = _("Note"),
    )

    def get_iban(self):
        return str(self.int_code) + str(self.cin) + str(self.abi) + str(self.cab) + str(self.cc)
    iban = property(get_iban)

    def get_detail_bank(self):
        try:
            dbank = DetailBank.objects.get(abi=self.abi, cab=self.cab)
            return dbank
        except:
            return None

    class Meta:
        verbose_name = _("bank")
        verbose_name_plural = _("banks")

    def __unicode__(self):
        return u"%s" % self.iban


#########################
# IVA                   #
#########################
class Vat(DateModel, UserModel, StatusModel, OrderedModel):
    ## ForeignKey
    ## Campi
    code = models.CharField(
        unique=True,
        max_length=15,
        verbose_name=_('Code'),
    )
    name = models.CharField(
        unique=True,
        max_length=100,
        verbose_name=_('Name'),
    )
    description = models.TextField(
        blank=True,
        null=True,
        verbose_name=_('Description'),
    )
    aliquot_perc = models.DecimalField(
        max_digits=4,
        decimal_places=2,
        default=0,
        verbose_name=_("aliquot perc"),
    )
    deduct_perc = models.DecimalField(
        max_digits=4,
        decimal_places=2,
        default=0,
        verbose_name=_("deduct perc"),
    )

    class Meta:
        verbose_name = _('VAT')
        verbose_name_plural = _('VATs')
        ordering = ('ordering', 'name')

    def __unicode__(self):
        return u"%s" % self.name

#########################
# VALUTA                #
#########################
class Currency (DateModel, UserModel, StatusModel, OrderedModel):
    ## ForeignKey
    ## Campi
    code = models.CharField(
        unique=True,
        max_length=15,
        verbose_name=_('Code'),
    )
    name = models.CharField(
        unique=True,
        max_length=100,
        verbose_name=_('Name'),
    )
    symbol = models.CharField(
        max_length=3,
        verbose_name=_('Symbol'),
    )

    class Meta:
        verbose_name = _('Currency')
        verbose_name_plural = _('Currencies')
        ordering = ('ordering', 'code')
        permissions = (
            ("view_currency"    , "Can view currency"),
        )

    def __unicode__(self):
        return u"%s" % self.name

#######################
# CONSEGNA            #
#######################
TRANSPORTBY_DEF = (
    ("x", "",),                 # None
    ("snd", _("Sender"),),      # mittente
    ("rcp", _("Recipient"),),   # destinatario
    ("vec", _("Vector"),),      # vector
)
#CARRIAGE_DEF = (
#    ("frw", _("CARRIAGE FORWARD"),),     # PORTO ASSEGNATO
#    ("pad", _("CARRIAGE PAID"),),        # PORTO FRANCO
#    ("exw", _("EX WORKS"),),             # EX WORKS
#    ("pdc", _("CARRIAGE PAID W/CHARGE")) # PORTO FRANCO CON ADDEBITO
#)
class Carriage(DateModel):
    code = models.CharField(
        max_length=3,
        primary_key = True,
        verbose_name = _("Code")
    )
    name = models.CharField(
        max_length = 50,
        verbose_name = _("Name")
    )
    description = models.TextField(
        blank = True, null = True,
        verbose_name = _("Description")
    )

    def __unicode__(self):
        return self.name


class Delivery (DateModel, UserModel, StatusModel, OrderedModel):
    ## ForeignKey
    ## Campi
    transport_by = models.CharField(
        max_length=3,
        choices=TRANSPORTBY_DEF,
        verbose_name=_('Transport by'),
    )
    carriage = models.ForeignKey(
        Carriage,
        verbose_name=_('Carriage'),
    )
    class Meta:
        verbose_name = _('Delivery')
        verbose_name_plural = _('Deliveries')
        ordering = ('ordering', 'transport_by', 'carriage')

    def __unicode__(self):
        return u"%(transport_by)s - %(carriage)s" % {'transport_by': self.get_transport_by_display(), 'carriage': self.carriage}

#############################
# AGENTI                    #
#############################
class Agent(DateModel, UserModel, StatusModel, OrderedModel):
    ## ForeignKey
    user = models.OneToOneField (
        User,
        blank = True, null = True,
        db_index = True,
        related_name='agent',
        verbose_name = _("User"),
    )
    organizations = models.ManyToManyField(
        Organization,
        through = 'Commision',
        related_name = "agent",
        verbose_name = _("Organizations")
    )
    ## Campi Modello
    code = models.CharField(
        unique = True,
        max_length = 15,
        db_index = True,
        verbose_name = _("code"),
    )
    name = models.CharField(
        max_length = 100,
        db_index = True,
        verbose_name = _('Name'),
    )
    email = models.EmailField (
        max_length = 100,
        blank = True, null = True,
        verbose_name = _("Email"),
    )
    mobile = models.CharField (
        max_length = 20,
        blank = True, null = True,
        verbose_name = _("Mobile"),
    )
    area = models.CharField(
        max_length = 100,
        blank = True, null = True,
        verbose_name = _("Area"),
    )
    is_area_manager = models.BooleanField(
        default = False,
        verbose_name = _("Is Area Manager"),
    )
    commision = models.DecimalField(
        max_digits=4,
        decimal_places=2,
        default=0,
        verbose_name=_("Commision %"),
        help_text = _("Default commision for the agent. This value will be used if Commision's value is empty")
    )

    class Meta:
        verbose_name = _('Agent')
        verbose_name_plural = _('Agents')
        permissions = (
            ("view_agent"    , "Can view agent"),
        )

    def __unicode__(self):
        return u'%s %s' % (self.code, self.name)

    def active_organizations(self):
        return self.organizations.filter(status=1)

    def disactive_organizations(self):
        return self.organizations.filter(status=1)

#############################
# PROVVIGIONI               #
#############################
class Commision(DateModel, UserModel, StatusModel, OrderedModel):
    agent = models.ForeignKey(
        Agent,
        related_name = "commisions",
        verbose_name = _("Agent")
    )
    organization = models.ForeignKey(
        Organization,
        related_name = "commisions",
        verbose_name = _("Organization")
    )
    commision = models.DecimalField(
        max_digits=4,
        decimal_places=2,
        default=0,
        verbose_name=_("Commision %"),
    )

    class Meta:
        verbose_name = _('Commision')
        verbose_name_plural = _('Commisions')
        unique_together = (('agent','organization',),)
        permissions = (
            ("view_commision"    , "Can view commision"),
        )

    def get_commision(self):
        commision = self.commision
        if not commision:
            commision = self.agent.commision
        return commision

    def __unicode__(self):
        commision = self.get_commision()
        return u'%s - %s' % (self.agent, commision)
